/**
 * Module Dependencies
 *
 * @ignore
 */
const uuidV4 = require('uuid/v4')
const winston = require('winston')

const config = {
  colorize: true,
  level: (process.env.LOG_LEVEL) ? process.env.LOG_LEVEL : 'debug'
}

if (process.env.NODE_ENV === 'production') {
  config.colorize = false
  config.json = true
  config.stringify = true
  config.timestamp = true
}

const logger = new winston.Logger({

  transports:[
    new winston.transports.Console(config)
  ]

})

class Logger {

  constructor(options = {}) {
    this.applicationName = options.applicationName || 'structure'
    this.createdAt = options.createdAt || new Date().toISOString()
    this.namespace = options.namespace || 'default'
    this.requestId = options.requestId || uuidV4()

    this.info('Logs started', {
      createdAt: this.createdAt
    })
  }

  child(namespace) {

    if(!namespace) {
      throw new Error('A child requires a name')
    }

    return new Logger({
      applicationName: this.applicationName,
      createdAt: new Date().toISOString(),
      namespace,
      requestId: this.requestId
    })

  }

  debug(message = '', data = {}) {
    this.log('debug', message, data)
  }

  end() {

    this.info('Logs completed', {
      createdAt: this.createdAt,
      endedAt: new Date().toISOString()
    })

  }

  error(message = '', data = {}) {
    this.log('error', message, data)
  }

  info(message = '', data = {}) {
    this.log('info', message, data)
  }

  log(level = '', message = '', data = {}) {

    data.applicationName = this.applicationName
    data.namespace = this.namespace
    data.requestId = this.requestId

    logger[level](message, data)
  }

  set(key, val) {
    this[key] = val
  }

  warn(message = '', data = {}) {
    this.log('warn', message, data)
  }

}

module.exports = Logger
